#include "mainwnd.h"
#include "resource.h"
#include <afxres.h>
#include "aboutdlg.h"
#include "application.h"
#include <commdlg.h>	// GetOpenFileName()
#include <stdlib.h>		// _MAX_PATH
#include <cderr.h>		// error codes for CommDlgExtendedError()

CMainWnd::CMainWnd() : edit()
{
}

int CMainWnd::Create()
{
	HICON hIcon = (HICON)LoadImage(GetModuleHandle(0), MAKEINTRESOURCE(IDI_ANGELCODE), IMAGE_ICON, LR_DEFAULTSIZE, LR_DEFAULTSIZE, LR_SHARED);
	HICON hIconSmall = (HICON)LoadImage(GetModuleHandle(0), MAKEINTRESOURCE(IDI_ANGELCODE), IMAGE_ICON, 16, 16, LR_SHARED);

	int r = RegisterClass("MainWnd", 0, 0, hIcon, hIconSmall, AC_REGDEFCURSOR);
	if( r < 0 ) return r;

	RECT rc = {0, 0, 320, 240};
	AdjustWindowRect(&rc, WS_OVERLAPPEDWINDOW, TRUE);

	r = acCWindow::Create("AngelCode Texture Generator", rc.right-rc.left, rc.bottom-rc.top, WS_OVERLAPPEDWINDOW, 0, 0, "MainWnd");
	if( r < 0 ) return r;

	SetMenu(MAKEINTRESOURCE(IDR_MENU));
	
	SetAccelerator(MAKEINTRESOURCE(IDR_ACCEL));

	ShowWindow(hWnd, SW_SHOW);

	return 0;
}

LRESULT CMainWnd::MsgProc(UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch( msg )
	{
	case WM_CREATE:
		{
			UINT style = ES_MULTILINE | 
				WS_VISIBLE | 
				WS_CHILD |
				ES_AUTOHSCROLL |
				ES_AUTOVSCROLL |
				WS_HSCROLL |
				WS_VSCROLL |
				0 /*ES_READONLY*/;
			RECT rc = {0, 0, 320, 240};
			edit.Create(style, &rc, this, 0);

			hFont = CreateFont(10,0,0, 0,0,0, 0,0,0, 0,0,0, 0, "Courier");
			SendMessage(edit.GetHandle(), WM_SETFONT, (WPARAM)hFont, TRUE);
		}
		return 0;

	case WM_SIZE:
		if( wParam != SIZE_MINIMIZED )
		{
			UINT w = LOWORD(lParam);
			UINT h = HIWORD(lParam);
			MoveWindow(edit.GetHandle(), 0, 0, w, h, TRUE);
		}
		return 0;

	case WM_DESTROY:
		DestroyWindow(edit.GetHandle());
		DeleteObject(hFont);

		PostQuitMessage(0);

		return 0;

	case WM_CLOSE:
		OnClose();
		return 0;

	case WM_COMMAND:
		switch( LOWORD(wParam) )
		{
		case ID_APP_EXIT:
			OnClose();
			return 0;

		case ID_APP_ABOUT:
			OnAbout();
			return 0;

		case ID_FILE_OPEN:
			OnFileOpen();
			return 0;

		case ID_SCRIPT_RUN:
			app.ExecuteScript();
			return 0;

		case ID_SCRIPT_STOP:
			app.AbortWork();
			return 0;

		case ID_SCRIPT_BUILD:
			app.CompileScript();
			return 0;
		}
		break;

	case WM_INITMENUPOPUP:
		OnInitMenuPopup((HMENU)wParam, LOWORD(lParam), HIWORD(lParam));
		return 0;

	case MSG_SHOWIMAGE:
		app.OpenImageWindow((int)wParam);
		return 0;
	}

	return DefWndProc(msg, wParam, lParam);
}

void CMainWnd::OnFileOpen()
{
	OPENFILENAME ofn;
	char file[_MAX_PATH] = "";

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hWnd;
	ofn.lpstrFilter = "AngelScript files\0*.AS\0";
	ofn.lpstrFile = file;
	ofn.nMaxFile = _MAX_PATH;
	ofn.Flags = OFN_FILEMUSTEXIST|OFN_HIDEREADONLY|OFN_NONETWORKBUTTON|OFN_PATHMUSTEXIST;
	ofn.lpstrDefExt = "as";

	if( GetOpenFileName(&ofn) != 0 )
	{
		// Set new file path
		app.SetScript(file);
	}
/*	else
	{
		DWORD dw = CommDlgExtendedError();
		switch( dw )
		{
		case CDERR_DIALOGFAILURE:
			OutputDebugString("CDERR_DIALOGFAILURE\n");
			break;
		case CDERR_FINDRESFAILURE:
			OutputDebugString("CDERR_FINDRESFAILURE\n");
			break;
		case CDERR_INITIALIZATION:
			OutputDebugString("CDERR_INITIALIZATION\n");
			break;
		case CDERR_LOADRESFAILURE:
			OutputDebugString("CDERR_LOADRESFAILURE\n");
			break;
		case CDERR_LOADSTRFAILURE:
			OutputDebugString("CDERR_LOADSTRFAILURE\n");
			break;
		case CDERR_LOCKRESFAILURE:
			OutputDebugString("CDERR_LOCKRESFAILURE\n");
			break;
		case CDERR_MEMALLOCFAILURE:
			OutputDebugString("CDERR_MEMALLOCFAILURE\n");
			break;
		case CDERR_MEMLOCKFAILURE:
			OutputDebugString("CDERR_MEMLOCKFAILURE\n");
			break;
		case CDERR_NOHINSTANCE:
			OutputDebugString("CDERR_NOHINSTANCE\n");
			break;
		case CDERR_NOHOOK:
			OutputDebugString("CDERR_NOHOOK\n");
			break;
		case CDERR_NOTEMPLATE:
			OutputDebugString("CDERR_NOTEMPLATE\n");
			break;
		case CDERR_REGISTERMSGFAIL:
			OutputDebugString("CDERR_REGISTERMSGFAIL\n");
			break;
		case CDERR_STRUCTSIZE:
			OutputDebugString("CDERR_STRUCTSIZE\n");
			break;
		case FNERR_BUFFERTOOSMALL:
			OutputDebugString("FNERR_BUFFERTOOSMALL\n");
			break;
		case FNERR_INVALIDFILENAME:
			OutputDebugString("FNERR_INVALIDFILENAME\n");
			break;
		case FNERR_SUBCLASSFAILURE:
			OutputDebugString("FNERR_SUBCLASSFAILURE\n");
			break;
		}
	}*/
}

void CMainWnd::OnClose()
{
	if( app.IsWorking() )
	{
		MessageBox(hWnd, "You can't close the application while it's working. Please stop the work from the Script menu before closing.", "Info", MB_OK|MB_ICONINFORMATION );
		return;
	}

	DestroyWindow(hWnd);
}

void CMainWnd::OnAbout()
{
	CAboutDlg dlg;

	dlg.DoDialog(this);
}

void CMainWnd::OnInitMenuPopup(HMENU hMenu, int pos, BOOL isWindowMenu)
{
	if( pos == 0 )
	{
		if( app.IsWorking() )
		{
			EnableMenuItem(hMenu, ID_FILE_OPEN, MF_BYCOMMAND | MF_GRAYED);
		}
		else
		{
			EnableMenuItem(hMenu, ID_FILE_OPEN, MF_BYCOMMAND | MF_ENABLED);
		}
	}
	else if( pos == 1 )
	{
		if( app.IsWorking() )
		{
			EnableMenuItem(hMenu, ID_SCRIPT_RUN, MF_BYCOMMAND | MF_GRAYED);
			EnableMenuItem(hMenu, ID_SCRIPT_BUILD, MF_BYCOMMAND | MF_GRAYED);
		}
		else
		{
			EnableMenuItem(hMenu, ID_SCRIPT_RUN, MF_BYCOMMAND | MF_ENABLED);
			EnableMenuItem(hMenu, ID_SCRIPT_BUILD, MF_BYCOMMAND | MF_ENABLED);
		}

		if( app.IsExecuting() )
		{
			EnableMenuItem(hMenu, ID_SCRIPT_STOP, MF_BYCOMMAND | MF_ENABLED);
		}
		else
		{
			EnableMenuItem(hMenu, ID_SCRIPT_STOP, MF_BYCOMMAND | MF_GRAYED);
		}
	}
}
