#include <windows.h>
#include <assert.h>
#include <mmsystem.h>
#include <angelscript.h>
#include <new.h>
#include "rand.h"

CSimpleRandomizer::CSimpleRandomizer()
{
	refCount = 1;
	randomize();
}

CSimpleRandomizer::~CSimpleRandomizer()
{
	assert( refCount == 0 );
}

void CSimpleRandomizer::AddRef()
{
	refCount++;
}

void CSimpleRandomizer::Release()
{
	if( --refCount == 0 )
		delete this;
}

CSimpleRandomizer &CSimpleRandomizer::operator=(const CSimpleRandomizer &other)
{
	randSeed = other.randSeed;
	return *this;
}

void CSimpleRandomizer::setSeed(unsigned int s)
{
	randSeed = s;
}

float CSimpleRandomizer::getNumber()
{
	randSeed = (69069 * randSeed + 1);
	return float(randSeed & 0x7fff)/float(0x8000);
}

void CSimpleRandomizer::randomize()
{
	// Set the seed to the time since system start
	randSeed = timeGetTime();

	// Get a random number a couple of times to shuffle the numbers
	getNumber();
	getNumber();
}

unsigned int CSimpleRandomizer::getSeed()
{
	return randSeed;
}

void SimpleRandomizer_Construct(CSimpleRandomizer *r)
{
	new(r) CSimpleRandomizer();
}

void RegisterScriptRandomizer(asIScriptEngine *engine)
{
	int r;

	r = engine->RegisterObjectType("randomizer", sizeof(CSimpleRandomizer), asOBJ_CLASS_CDA); assert( r >= 0 );

	r = engine->RegisterObjectBehaviour("randomizer", asBEHAVE_CONSTRUCT, "void f()", asFUNCTION(SimpleRandomizer_Construct), asCALL_CDECL_OBJLAST); assert( r >= 0 );
	r = engine->RegisterObjectBehaviour("randomizer", asBEHAVE_ADDREF, "void f()", asMETHOD(CSimpleRandomizer, AddRef), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectBehaviour("randomizer", asBEHAVE_RELEASE, "void f()", asMETHOD(CSimpleRandomizer, Release), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectBehaviour("randomizer", asBEHAVE_ASSIGNMENT, "randomizer &f(const randomizer &in)", asMETHOD(CSimpleRandomizer, operator=), asCALL_THISCALL); assert( r >= 0 );

	r = engine->RegisterObjectMethod("randomizer", "void randomize()", asMETHOD(CSimpleRandomizer, randomize), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("randomizer", "void setSeed(uint)", asMETHOD(CSimpleRandomizer, setSeed), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("randomizer", "uint getSeed(uint)", asMETHOD(CSimpleRandomizer, getSeed), asCALL_THISCALL); assert( r >= 0 );
	r = engine->RegisterObjectMethod("randomizer", "float getNumber()", asMETHOD(CSimpleRandomizer, getNumber), asCALL_THISCALL); assert( r >= 0 );
}